@@ -17,6 +17,7 @@ from typing import (
 
 from rasa.cli import utils as cli_utils
 from rasa.shared.constants import DOCS_BASE_URL, DEFAULT_SENDER_ID
+from rasa.shared.exceptions import RasaException
 
 try:
     from urlparse import urljoin  # pytype: disable=import-error
@@ -117,7 +118,7 @@ class InputChannel:
 
     @classmethod
     def raise_missing_credentials_exception(cls) -> NoReturn:
-        raise Exception(
+        raise RasaException(
             f"To use the {cls.name()} input channel, you need to "
             f"pass a credentials file using '--credentials'. "
             f"The argument should be a file path pointing to "
