@@ -39,9 +39,15 @@ def find_unavailable_packages(package_names: List[Text]) -> Set[Text]:
     return failed_imports
 
 
-def validate_requirements(component_names: List[Text]) -> None:
+def validate_requirements(component_names: List[Optional[Text]]) -> None:
     """Validates that all required importable python packages are installed.
 
+    Raises:
+        InvalidConfigError: If one of the component names is `None`, likely
+            indicates that a custom implementation is missing this property
+            or that there is an invalid configuration file that we did not
+            catch earlier.
+
     Args:
         component_names: The list of component names.
     """
@@ -51,6 +57,13 @@ def validate_requirements(component_names: List[Text]) -> None:
     # Validate that all required packages are installed
     failed_imports = {}
     for component_name in component_names:
+        if component_name is None:
+            raise InvalidConfigError(
+                "Your pipeline configuration contains a component that is missing "
+                "a name. Please double check your configuration or if this is a "
+                "custom component make sure to implement the name property for "
+                "the component."
+            )
         component_class = registry.get_component_class(component_name)
         unavailable_packages = find_unavailable_packages(
             component_class.required_packages()
@@ -782,7 +795,7 @@ class ComponentBuilder:
                 self.__add_to_cache(component, cache_key)
             return component
         except MissingArgumentError as e:  # pragma: no cover
-            raise Exception(
+            raise RasaException(
                 f"Failed to load component from file '{component_meta.get('file')}'. "
                 f"Error: {e}"
             )
@@ -815,7 +828,7 @@ class ComponentBuilder:
                 self.__add_to_cache(component, cache_key)
             return component
         except MissingArgumentError as e:  # pragma: no cover
-            raise Exception(
+            raise RasaException(
                 f"Failed to create component '{component_config['name']}'. "
                 f"Error: {e}"
             )
